@@ -19,17 +19,14 @@
 #include <QMap>
 #include <QMimeDatabase>
 
-#include <KAuthorized>
 #include <KConfigGroup>
 #include <KDesktopFile>
-#include <KLocalizedString>
 #include <KShell>
 
 #include <QDebug>
 #include <QStandardPaths>
 
 #include "kservicefactory_p.h"
-#include "kservicetypefactory_p.h"
 #include "kserviceutil_p.h"
 #include "servicesdebug.h"
 
@@ -55,8 +52,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
 
     bool absPath = !QDir::isRelativePath(entryPath);
 
-    // TODO: it makes sense to have a KConstConfigGroup I guess
-    const KConfigGroup desktopGroup = const_cast<KDesktopFile *>(config)->desktopGroup();
+    const KConfigGroup desktopGroup = config->desktopGroup();
     QMap<QString, QString> entryMap = desktopGroup.entryMap();
 
     entryMap.remove(QStringLiteral("Encoding")); // reserved as part of Desktop Entry Standard
@@ -82,14 +78,9 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
         }
     }
 
-    m_strType = config->readType();
-    entryMap.remove(QStringLiteral("Type"));
+    m_strType = entryMap.take(QStringLiteral("Type"));
     if (m_strType.isEmpty()) {
-        /*kWarning(servicesDebugArea()) << "The desktop entry file " << entryPath
-          << " has no Type=... entry."
-          << " It should be \"Application\" or \"Service\"";
-          m_bValid = false;
-          return;*/
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "does not have a \"Type=Application\" set.";
         m_strType = QStringLiteral("Application");
     } else if (m_strType != QLatin1String("Application") && m_strType != QLatin1String("Service")) {
         qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "instead of \"Application\" or \"Service\"";
@@ -101,17 +92,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     // various expansions, like $HOME.  Note that the expansion
     // behaviour still happens if the "e" flag is set, maintaining
     // backwards compatibility.
-    m_strExec = desktopGroup.readEntry("Exec", QString());
-    entryMap.remove(QStringLiteral("Exec"));
-
-    if (m_strType == QLatin1String("Application")) {
-        // It's an application? Should have an Exec line then, otherwise we can't run it
-        if (m_strExec.isEmpty()) {
-            qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "but no Exec line";
-            m_bValid = false;
-            return;
-        }
-    }
+    m_strExec = entryMap.take(QStringLiteral("Exec"));
 
     // In case Try Exec is set, check if the application is available
     if (!config->tryExec()) {
@@ -129,33 +110,21 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
         return;
     }
 
-    if ((m_strType == QLatin1String("Service")) && (locationType != QStandardPaths::GenericDataLocation) && !absPath) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType << "but is located under \""
-                            << QStandardPaths::displayName(locationType) << "\" instead of \"Shared Data\"/kservices5";
-        m_bValid = false;
-        return;
-    }
-
     // entryPath To desktopEntryName
-    // (e.g. "/home/x/.qttest/share/kservices5/fakepart2.desktop" --> "fakepart2")
+    // (e.g. "/usr/share/applications/org.kde.kate" --> "org.kde.kate")
     QString _name = KServiceUtilPrivate::completeBaseName(entryPath);
 
-    m_strIcon = config->readIcon();
-    entryMap.remove(QStringLiteral("Icon"));
-    m_bTerminal = desktopGroup.readEntry("Terminal", false); // should be a property IMHO
+    m_strIcon = entryMap.take(QStringLiteral("Icon"));
+    m_bTerminal = desktopGroup.readEntry("Terminal", false);
     entryMap.remove(QStringLiteral("Terminal"));
-    m_strTerminalOptions = desktopGroup.readEntry("TerminalOptions"); // should be a property IMHO
-    entryMap.remove(QStringLiteral("TerminalOptions"));
-    m_strWorkingDirectory = KShell::tildeExpand(config->readPath());
-    entryMap.remove(QStringLiteral("Path"));
-    m_strComment = config->readComment();
-    entryMap.remove(QStringLiteral("Comment"));
-    m_strGenName = config->readGenericName();
-    entryMap.remove(QStringLiteral("GenericName"));
-    QString _untranslatedGenericName = desktopGroup.readEntryUntranslated("GenericName");
-    if (!_untranslatedGenericName.isEmpty()) {
-        entryMap.insert(QStringLiteral("UntranslatedGenericName"), _untranslatedGenericName);
-    }
+    m_strTerminalOptions = entryMap.take(QStringLiteral("TerminalOptions"));
+    m_strWorkingDirectory = KShell::tildeExpand(entryMap.take(QStringLiteral("Path")));
+    m_strComment = entryMap.take(QStringLiteral("Comment"));
+    m_strGenName = entryMap.take(QStringLiteral("GenericName"));
+
+    // Store these as member variables too, because the lookup will be significanly faster
+    m_untranslatedGenericName = desktopGroup.readEntryUntranslated("GenericName");
+    m_untranslatedName = desktopGroup.readEntryUntranslated("Name");
 
     m_lstFormFactors = desktopGroup.readEntry("X-KDE-FormFactors", QStringList());
     entryMap.remove(QStringLiteral("X-KDE-FormFactors"));
@@ -166,31 +135,10 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     entryMap.remove(QStringLiteral("X-KDE-Keywords"));
     categories = desktopGroup.readXdgListEntry("Categories");
     entryMap.remove(QStringLiteral("Categories"));
-    // TODO KDE5: only care for X-KDE-Library in Type=Service desktop files
-    // This will prevent people defining a part and an app in the same desktop file
-    // which makes user-preference handling difficult.
-    m_strLibrary = desktopGroup.readEntry("X-KDE-Library");
-    entryMap.remove(QStringLiteral("X-KDE-Library"));
-    if (!m_strLibrary.isEmpty() && m_strType == QLatin1String("Application")) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has Type=" << m_strType
-                            << "but also has a X-KDE-Library key. This works for now,"
-                               " but makes user-preference handling difficult, so support for this might"
-                               " be removed at some point. Consider splitting it into two desktop files.";
-    }
 
-    QStringList lstServiceTypes = desktopGroup.readEntry("ServiceTypes", QStringList());
-    entryMap.remove(QStringLiteral("ServiceTypes"));
-    lstServiceTypes += desktopGroup.readEntry("X-KDE-ServiceTypes", QStringList());
-    entryMap.remove(QStringLiteral("X-KDE-ServiceTypes"));
-    lstServiceTypes += desktopGroup.readXdgListEntry("MimeType");
+    const QStringList lstServiceTypes = desktopGroup.readXdgListEntry("MimeType");
     entryMap.remove(QStringLiteral("MimeType"));
 
-    if (m_strType == QLatin1String("Application") && !lstServiceTypes.contains(QLatin1String("Application")))
-    // Applications implement the service type "Application" ;-)
-    {
-        lstServiceTypes += QStringLiteral("Application");
-    }
-
     m_initialPreference = desktopGroup.readEntry("InitialPreference", 1);
     entryMap.remove(QStringLiteral("InitialPreference"));
 
@@ -217,49 +165,11 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
         m_serviceTypes.push_back(KService::ServiceTypeAndPreference(initialPreference, st));
     }
 
-    if (entryMap.contains(QLatin1String("Actions"))) {
-        parseActions(config, q);
-    }
-
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 102)
-    QString dbusStartupType = desktopGroup.readEntry("X-DBUS-StartupType").toLower();
-    entryMap.remove(QStringLiteral("X-DBUS-StartupType"));
-    if (dbusStartupType == QLatin1String("unique")) {
-        m_DBUSStartusType = KService::DBusUnique;
-    } else if (dbusStartupType == QLatin1String("multi")) {
-        m_DBUSStartusType = KService::DBusMulti;
-    } else {
-        m_DBUSStartusType = KService::DBusNone;
-    }
-#endif
-
     m_strDesktopEntryName = _name;
 
-    // Exec lines from the KCMs always have the pattern "<program> m_strDesktopEntryName", see https://phabricator.kde.org/T13729
-    const static bool hasSystemsettings = !QStandardPaths::findExecutable(QStringLiteral("systemsettings5")).isEmpty();
-    if (m_strExec.isEmpty() && serviceTypes().contains(QStringLiteral("KCModule"))) {
-        if (desktopGroup.readEntry("X-KDE-ParentApp") == QLatin1String("kinfocenter")) {
-            m_strExec = QStringLiteral("kinfocenter ") + m_strDesktopEntryName;
-        } else if (desktopGroup.readEntry("X-KDE-ParentApp") == QLatin1String("kcontrol")) {
-            if (!desktopGroup.readEntry("X-KDE-System-Settings-Parent-Category").isEmpty() && hasSystemsettings) {
-                m_strExec = QStringLiteral("systemsettings5 ") + m_strDesktopEntryName;
-            } else {
-                m_strExec = QStringLiteral("kcmshell5 ") + m_strDesktopEntryName;
-            }
-        }
-    }
     m_bAllowAsDefault = desktopGroup.readEntry("AllowDefault", true);
     entryMap.remove(QStringLiteral("AllowDefault"));
 
-    // allow plugin users to translate categories without needing a separate key
-    auto entryIt = entryMap.find(QStringLiteral("X-KDE-PluginInfo-Category"));
-    if (entryIt != entryMap.end()) {
-        const QString &key = entryIt.key();
-        m_mapProps.insert(key, QVariant(desktopGroup.readEntryUntranslated(key)));
-        m_mapProps.insert(key + QLatin1String("-Translated"), QVariant(entryIt.value()));
-        entryMap.erase(entryIt);
-    }
-
     // Store all additional entries in the property map.
     // A QMap<QString,QString> would be easier for this but we can't
     // break BC, so we have to store it in m_mapProps.
@@ -267,6 +177,12 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     auto it = entryMap.constBegin();
     for (; it != entryMap.constEnd(); ++it) {
         const QString key = it.key();
+
+        // Ignore Actions, we parse that below
+        if (key == QLatin1String("Actions")) {
+            continue;
+        }
+
         // do not store other translations like Name[fr]; kbuildsycoca will rerun if we change languages anyway
         if (!key.contains(QLatin1Char('['))) {
             // qCDebug(SERVICES) << "  Key =" << key << " Data =" << it.value();
@@ -277,6 +193,12 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
             }
         }
     }
+
+    // parse actions last since that may clone the service
+    // we want all other information parsed by then
+    if (entryMap.contains(QLatin1String("Actions"))) {
+        parseActions(config, q);
+    }
 }
 
 void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
@@ -321,6 +243,8 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
             qCWarning(SERVICES) << "The desktop file" << q->entryPath() << "references the action" << group << "but doesn't define it";
         }
     }
+
+    serviceClone->setActions(m_actions);
 }
 
 void KServicePrivate::load(QDataStream &s)
@@ -344,14 +268,12 @@ void KServicePrivate::load(QDataStream &s)
       >> initpref
       >> m_lstKeywords >> m_strGenName
       >> categories >> menuId >> m_actions >> m_serviceTypes
-      >> m_lstFormFactors;
+      >> m_lstFormFactors
+      >> m_untranslatedName >> m_untranslatedGenericName;
     // clang-format on
 
     m_bAllowAsDefault = bool(def);
     m_bTerminal = bool(term);
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 102)
-    m_DBUSStartusType = static_cast<KService::DBusStartupType>(dst);
-#endif
     m_initialPreference = initpref;
 
     m_bValid = true;
@@ -363,11 +285,7 @@ void KServicePrivate::save(QDataStream &s)
     qint8 def = m_bAllowAsDefault;
     qint8 initpref = m_initialPreference;
     qint8 term = m_bTerminal;
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 102)
-    qint8 dst = qint8(m_DBUSStartusType);
-#else
     qint8 dst = 0;
-#endif
 
     // WARNING: THIS NEEDS TO REMAIN COMPATIBLE WITH PREVIOUS KService 5.x VERSIONS!
     // !! This data structure should remain binary compatible at all times !!
@@ -375,7 +293,7 @@ void KServicePrivate::save(QDataStream &s)
     // number in ksycoca.cpp
     s << m_strType << m_strName << m_strExec << m_strIcon << term << m_strTerminalOptions << m_strWorkingDirectory << m_strComment << def << m_mapProps
       << m_strLibrary << dst << m_strDesktopEntryName << initpref << m_lstKeywords << m_strGenName << categories << menuId << m_actions << m_serviceTypes
-      << m_lstFormFactors;
+      << m_lstFormFactors << m_untranslatedName << m_untranslatedGenericName;
 }
 
 ////
@@ -429,49 +347,6 @@ KService::~KService()
 {
 }
 
-bool KService::hasServiceType(const QString &serviceType) const
-{
-    Q_D(const KService);
-
-    if (!d->m_bValid) {
-        return false; // (useless) safety test
-    }
-    const KServiceType::Ptr ptr = KServiceType::serviceType(serviceType);
-    if (!ptr) {
-        return false;
-    }
-    const int serviceOffset = offset();
-    // doesn't seem to work:
-    // if ( serviceOffset == 0 )
-    //    serviceOffset = serviceByStorageId( storageId() );
-    if (serviceOffset) {
-        KSycoca::self()->ensureCacheValid();
-        return KSycocaPrivate::self()->serviceFactory()->hasOffer(ptr->offset(), ptr->serviceOffersOffset(), serviceOffset);
-    }
-
-    // fall-back code for services that are NOT from ksycoca
-    // For each service type we are associated with, if it doesn't
-    // match then we try its parent service types.
-    const QString serviceTypeName = ptr->name();
-    auto matchFunc = [&serviceTypeName](const ServiceTypeAndPreference &typePref) {
-        const QString &st = typePref.serviceType;
-        // qCDebug(SERVICES) << "    has " << typePref;
-        if (st == serviceTypeName) {
-            return true;
-        }
-
-        // also the case of parent servicetypes
-        KServiceType::Ptr p = KServiceType::serviceType(st);
-        if (p && p->inherits(serviceTypeName)) {
-            return true;
-        }
-
-        return false;
-    };
-
-    return std::any_of(d->m_serviceTypes.cbegin(), d->m_serviceTypes.cend(), matchFunc);
-}
-
 bool KService::hasMimeType(const QString &mimeType) const
 {
     Q_D(const KService);
@@ -508,67 +383,60 @@ bool KService::hasMimeType(const QString &mimeType) const
     return std::any_of(d->m_serviceTypes.cbegin(), d->m_serviceTypes.cend(), matchFunc);
 }
 
-QVariant KServicePrivate::property(const QString &_name) const
-{
-    return property(_name, QMetaType::UnknownType);
-}
-
-// Return a string QVariant if string isn't null, and invalid variant otherwise
-// (the variant must be invalid if the field isn't in the .desktop file)
-// This allows trader queries like "exist Library" to work.
-static QVariant makeStringVariant(const QString &string)
-{
-    // Using isEmpty here would be wrong.
-    // Empty is "specified but empty", null is "not specified" (in the .desktop file)
-    return string.isNull() ? QVariant() : QVariant(string);
-}
-
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 102)
-QVariant KService::property(const QString &_name, QVariant::Type t) const
-{
-    Q_D(const KService);
-    return d->property(_name, (QMetaType::Type)t);
-}
-#endif
-
 QVariant KService::property(const QString &_name, QMetaType::Type t) const
 {
     Q_D(const KService);
     return d->property(_name, t);
 }
 
+template<>
+QString KService::property<QString>(const QString &_name) const
+{
+    Q_D(const KService);
+
+    if (_name == QLatin1String("Type")) {
+        return d->m_strType;
+    } else if (_name == QLatin1String("Name")) {
+        return d->m_strName;
+    } else if (_name == QLatin1String("Exec")) {
+        return d->m_strExec;
+    } else if (_name == QLatin1String("Icon")) {
+        return d->m_strIcon;
+    } else if (_name == QLatin1String("TerminalOptions")) {
+        return d->m_strTerminalOptions;
+    } else if (_name == QLatin1String("Path")) {
+        return d->m_strWorkingDirectory;
+    } else if (_name == QLatin1String("Comment")) {
+        return d->m_strComment;
+    } else if (_name == QLatin1String("GenericName")) {
+        return d->m_strGenName;
+    } else if (_name == QLatin1String("DesktopEntryPath")) {
+        return d->path;
+    } else if (_name == QLatin1String("DesktopEntryName")) {
+        return d->m_strDesktopEntryName;
+    } else if (_name == QLatin1String("UntranslatedName")) {
+        return d->m_untranslatedName;
+    } else if (_name == QLatin1String("UntranslatedGenericName")) {
+        return d->m_untranslatedGenericName;
+    }
+
+    auto it = d->m_mapProps.constFind(_name);
+
+    if (it != d->m_mapProps.cend()) {
+        return it.value().toString();
+    }
+
+    return QString();
+}
+
 QVariant KServicePrivate::property(const QString &_name, QMetaType::Type t) const
 {
-    if (_name == QLatin1String("Type")) {
-        return QVariant(m_strType); // can't be null
-    } else if (_name == QLatin1String("Name")) {
-        return QVariant(m_strName); // can't be null
-    } else if (_name == QLatin1String("Exec")) {
-        return makeStringVariant(m_strExec);
-    } else if (_name == QLatin1String("Icon")) {
-        return makeStringVariant(m_strIcon);
-    } else if (_name == QLatin1String("Terminal")) {
+    if (_name == QLatin1String("Terminal")) {
         return QVariant(m_bTerminal);
-    } else if (_name == QLatin1String("TerminalOptions")) {
-        return makeStringVariant(m_strTerminalOptions);
-    } else if (_name == QLatin1String("Path")) {
-        return makeStringVariant(m_strWorkingDirectory);
-    } else if (_name == QLatin1String("Comment")) {
-        return makeStringVariant(m_strComment);
-    } else if (_name == QLatin1String("GenericName")) {
-        return makeStringVariant(m_strGenName);
-    } else if (_name == QLatin1String("ServiceTypes")) {
-        return QVariant(serviceTypes());
     } else if (_name == QLatin1String("AllowAsDefault")) {
         return QVariant(m_bAllowAsDefault);
     } else if (_name == QLatin1String("InitialPreference")) {
         return QVariant(m_initialPreference);
-    } else if (_name == QLatin1String("Library")) {
-        return makeStringVariant(m_strLibrary);
-    } else if (_name == QLatin1String("DesktopEntryPath")) { // can't be null
-        return QVariant(path);
-    } else if (_name == QLatin1String("DesktopEntryName")) {
-        return QVariant(m_strDesktopEntryName); // can't be null
     } else if (_name == QLatin1String("Categories")) {
         return QVariant(categories);
     } else if (_name == QLatin1String("Keywords")) {
@@ -577,67 +445,22 @@ QVariant KServicePrivate::property(const QString &_name, QMetaType::Type t) cons
         return QVariant(m_lstFormFactors);
     }
 
-    // Ok we need to convert the property from a QString to its real type.
-    // Maybe the caller helped us.
-    if (t == QMetaType::UnknownType) {
-        // No luck, let's ask KServiceTypeFactory what the type of this property
-        // is supposed to be.
-        // ######### this looks in all servicetypes, not just the ones this service supports!
-        KSycoca::self()->ensureCacheValid();
-        t = KSycocaPrivate::self()->serviceTypeFactory()->findPropertyTypeByName(_name);
-        if (t == QMetaType::UnknownType) {
-            qCDebug(SERVICES) << "Request for unknown property" << _name;
-            return QVariant(); // Unknown property: Invalid variant.
-        }
-    }
-
     auto it = m_mapProps.constFind(_name);
     if (it == m_mapProps.cend() || !it.value().isValid()) {
         // qCDebug(SERVICES) << "Property not found " << _name;
         return QVariant(); // No property set.
     }
 
-    if (t == QMetaType::QString) {
+    if (it->typeId() == t) {
         return it.value(); // no conversion necessary
     } else {
         // All others
         // For instance properties defined as StringList, like MimeTypes.
         // XXX This API is accessible only through a friend declaration.
-
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-        return KConfigGroup::convertToQVariant(_name.toUtf8().constData(), it.value().toString().toUtf8(), QVariant(static_cast<QVariant::Type>(t)));
-#else
         return KConfigGroup::convertToQVariant(_name.toUtf8().constData(), it.value().toString().toUtf8(), QVariant(QMetaType(t)));
-#endif
     }
 }
 
-QStringList KServicePrivate::propertyNames() const
-{
-    static const QStringList defaultKeys = {
-        QStringLiteral("Type"),
-        QStringLiteral("Name"),
-        QStringLiteral("Comment"),
-        QStringLiteral("GenericName"),
-        QStringLiteral("Icon"),
-        QStringLiteral("Exec"),
-        QStringLiteral("Terminal"),
-        QStringLiteral("TerminalOptions"),
-        QStringLiteral("Path"),
-        QStringLiteral("ServiceTypes"),
-        QStringLiteral("AllowAsDefault"),
-        QStringLiteral("InitialPreference"),
-        QStringLiteral("Library"),
-        QStringLiteral("DesktopEntryPath"),
-        QStringLiteral("DesktopEntryName"),
-        QStringLiteral("Keywords"),
-        QStringLiteral("FormFactors"),
-        QStringLiteral("Categories"),
-    };
-
-    return m_mapProps.keys() + defaultKeys;
-}
-
 KService::List KService::allServices()
 {
     KSycoca::self()->ensureCacheValid();
@@ -670,16 +493,13 @@ KService::Ptr KService::serviceByStorageId(const QString &_storageId)
 
 bool KService::substituteUid() const
 {
-    QVariant v = property(QStringLiteral("X-KDE-SubstituteUID"), QMetaType::Bool);
-    return v.isValid() && v.toBool();
+    return property<bool>(QStringLiteral("X-KDE-SubstituteUID"));
 }
 
 QString KService::username() const
 {
     // See also KDesktopFile::tryExec()
-    QString user;
-    QVariant v = property(QStringLiteral("X-KDE-Username"), QMetaType::QString);
-    user = v.isValid() ? v.toString() : QString();
+    QString user = property<QString>(QStringLiteral("X-KDE-Username"));
     if (user.isEmpty()) {
         user = QString::fromLocal8Bit(qgetenv("ADMIN_ACCOUNT"));
     }
@@ -695,21 +515,13 @@ bool KService::showInCurrentDesktop() const
 
     const QString envVar = QString::fromLatin1(qgetenv("XDG_CURRENT_DESKTOP"));
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-    QVector<QStringView> currentDesktops = QStringView(envVar).split(QLatin1Char(':'), Qt::SkipEmptyParts);
-#else
-    QVector<QStringRef> currentDesktops = envVar.splitRef(QLatin1Char(':'), Qt::SkipEmptyParts);
-#endif
+    QList<QStringView> currentDesktops = QStringView(envVar).split(QLatin1Char(':'), Qt::SkipEmptyParts);
 
     const QString kde = QStringLiteral("KDE");
     if (currentDesktops.isEmpty()) {
         // This could be an old display manager, or e.g. a failsafe session with no desktop name
         // In doubt, let's say we show KDE stuff.
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
         currentDesktops.append(kde);
-#else
-        currentDesktops.append(&kde);
-#endif
     }
 
     // This algorithm is described in the desktop entry spec
@@ -767,7 +579,7 @@ bool KService::showOnCurrentPlatform() const
 
 bool KService::noDisplay() const
 {
-    if (qvariant_cast<bool>(property(QStringLiteral("NoDisplay"), QMetaType::Bool))) {
+    if (property<bool>(QStringLiteral("NoDisplay"))) {
         return true;
     }
 
@@ -778,48 +590,20 @@ bool KService::noDisplay() const
     if (!showOnCurrentPlatform()) {
         return true;
     }
-
-    if (!KAuthorized::authorizeControlModule(storageId())) {
-        return true;
-    }
-
     return false;
 }
 
 QString KService::untranslatedGenericName() const
 {
-    QVariant v = property(QStringLiteral("UntranslatedGenericName"), QMetaType::QString);
-    return v.isValid() ? v.toString() : QString();
+    Q_D(const KService);
+    return d->m_untranslatedGenericName;
 }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 87)
-QString KService::parentApp() const
+QString KService::untranslatedName() const
 {
     Q_D(const KService);
-    auto it = d->m_mapProps.constFind(QStringLiteral("X-KDE-ParentApp"));
-    if (it != d->m_mapProps.cend()) {
-        const QVariant &val = it.value();
-        if (val.isValid()) {
-            return val.toString();
-        }
-    }
-
-    return {};
+    return d->m_untranslatedName;
 }
-#endif
-
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 87)
-QString KService::pluginKeyword() const
-{
-    Q_D(const KService);
-    QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("X-KDE-PluginKeyword"));
-    if ((it == d->m_mapProps.end()) || (!it->isValid())) {
-        return QString();
-    }
-
-    return it->toString();
-}
-#endif
 
 QString KService::docPath() const
 {
@@ -932,18 +716,9 @@ bool KService::isApplication() const
 QString KService::exec() const
 {
     Q_D(const KService);
-    if (d->m_strType == QLatin1String("Application") && d->m_strExec.isEmpty()) {
-        qCWarning(SERVICES) << "The desktop entry file" << entryPath() << "has Type=" << d->m_strType << "but has no Exec field.";
-    }
     return d->m_strExec;
 }
 
-QString KService::library() const
-{
-    Q_D(const KService);
-    return d->m_strLibrary;
-}
-
 QString KService::icon() const
 {
     Q_D(const KService);
@@ -964,10 +739,10 @@ bool KService::terminal() const
 
 bool KService::runOnDiscreteGpu() const
 {
-    QVariant prop = property(QStringLiteral("PrefersNonDefaultGPU"), QMetaType::Bool);
+    QVariant prop = property<bool>(QStringLiteral("PrefersNonDefaultGPU"));
     if (!prop.isValid()) {
         // For backwards compatibility
-        prop = property(QStringLiteral("X-KDE-RunOnDiscreteGpu"), QMetaType::Bool);
+        prop = property<bool>(QStringLiteral("X-KDE-RunOnDiscreteGpu"));
     }
 
     return prop.isValid() && prop.toBool();
@@ -979,22 +754,6 @@ QString KService::desktopEntryName() const
     return d->m_strDesktopEntryName;
 }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 102)
-KService::DBusStartupType KService::dbusStartupType() const
-{
-    Q_D(const KService);
-    return d->m_DBUSStartusType;
-}
-#endif
-
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 63)
-QString KService::path() const
-{
-    Q_D(const KService);
-    return d->m_strWorkingDirectory;
-}
-#endif
-
 QString KService::workingDirectory() const
 {
     Q_D(const KService);
@@ -1019,25 +778,6 @@ QStringList KService::keywords() const
     return d->m_lstKeywords;
 }
 
-QStringList KServicePrivate::serviceTypes() const
-{
-    QStringList ret;
-    ret.reserve(m_serviceTypes.size());
-
-    std::transform(m_serviceTypes.cbegin(), m_serviceTypes.cend(), std::back_inserter(ret), [](const KService::ServiceTypeAndPreference &typePref) {
-        Q_ASSERT(!typePref.serviceType.isEmpty());
-        return typePref.serviceType;
-    });
-
-    return ret;
-}
-
-QStringList KService::serviceTypes() const
-{
-    Q_D(const KService);
-    return d->serviceTypes();
-}
-
 QStringList KService::mimeTypes() const
 {
     Q_D(const KService);
@@ -1054,13 +794,29 @@ QStringList KService::mimeTypes() const
     return ret;
 }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 67)
-bool KService::allowAsDefault() const
+QStringList KService::supportedProtocols() const
 {
     Q_D(const KService);
-    return d->m_bAllowAsDefault;
+
+    QStringList ret;
+
+    const QLatin1String schemeHandlerPrefix("x-scheme-handler/");
+    for (const KService::ServiceTypeAndPreference &s : d->m_serviceTypes) {
+        const QString servType = s.serviceType;
+        if (servType.startsWith(schemeHandlerPrefix)) {
+            ret.append(servType.mid(schemeHandlerPrefix.size()));
+        }
+    }
+
+    const QStringList protocols = property<QStringList>(QStringLiteral("X-KDE-Protocols"));
+    for (const QString &protocol : protocols) {
+        if (!ret.contains(protocol)) {
+            ret.append(protocol);
+        }
+    }
+
+    return ret;
 }
-#endif
 
 int KService::initialPreference() const
 {
@@ -1100,9 +856,10 @@ void KService::setWorkingDirectory(const QString &workingDir)
     }
 }
 
-QVector<KService::ServiceTypeAndPreference> &KService::_k_accessServiceTypes()
+QList<KService::ServiceTypeAndPreference> KService::_k_accessServiceTypes()
 {
     Q_D(KService);
+
     return d->m_serviceTypes;
 }
 
@@ -1112,24 +869,28 @@ QList<KServiceAction> KService::actions() const
     return d->m_actions;
 }
 
-#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 86)
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 86)
-KService::operator KPluginName() const
-{
-    if (!isValid()) {
-        return KPluginName::fromErrorString(i18n("The provided service is not valid"));
-    }
-
-    if (library().isEmpty()) {
-        return KPluginName::fromErrorString(i18n("The service '%1' provides no library or the Library key is missing", entryPath()));
-    }
-
-    return KPluginName(library());
-}
-#endif
-#endif
-
 QString KService::aliasFor() const
 {
-    return KServiceUtilPrivate::completeBaseName(property(QStringLiteral("X-KDE-AliasFor"), QMetaType::QString).toString());
+    return KServiceUtilPrivate::completeBaseName(property<QString>(QStringLiteral("X-KDE-AliasFor")));
+}
+
+void KService::setActions(const QList<KServiceAction> &actions)
+{
+    Q_D(KService);
+    d->m_actions = actions;
+}
+
+std::optional<bool> KService::startupNotify() const
+{
+    Q_D(const KService);
+
+    if (QVariant value = d->m_mapProps.value(QStringLiteral("StartupNotify")); value.isValid()) {
+        return value.toBool();
+    }
+
+    if (QVariant value = d->m_mapProps.value(QStringLiteral("X-KDE-StartupNotify")); value.isValid()) {
+        return value.toBool();
+    }
+
+    return {};
 }
