@@ -16,7 +16,6 @@
 #include "kbuildmimetypefactory_p.h"
 #include "kbuildservicefactory_p.h"
 #include "kbuildservicegroupfactory_p.h"
-#include "kbuildservicetypefactory_p.h"
 #include "kctimefactory_p.h"
 #include <QDataStream>
 #include <QDateTime>
@@ -127,15 +126,6 @@ KService::Ptr KBuildSycoca::createService(const QString &path)
     return KService::Ptr(static_cast<KService *>(entry.data()));
 }
 
-static QStringList locateDirInResource(const QString &resourceSubdir)
-{
-    const QString dir = QStringLiteral(":/") + resourceSubdir; // e.g. :/kservicetypes5
-    if (QDir(dir).exists()) {
-        return {dir};
-    }
-    return {};
-}
-
 // returns false if the database is up to date, true if it needs to be saved
 bool KBuildSycoca::build()
 {
@@ -202,8 +192,7 @@ bool KBuildSycoca::build()
         m_resource = it1.value();
 
         QSet<QString> relFiles;
-        const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, m_resourceSubdir, QStandardPaths::LocateDirectory)
-            + locateDirInResource(m_resourceSubdir);
+        const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, m_resourceSubdir, QStandardPaths::LocateDirectory);
         qCDebug(SYCOCA) << "Looking for subdir" << m_resourceSubdir << "=>" << dirs;
         for (const QString &dir : dirs) {
             QDirIterator it(dir, QDirIterator::Subdirectories);
@@ -254,7 +243,7 @@ bool KBuildSycoca::build()
             m_vfolder->setTrackId(m_trackId);
         }
 
-        VFolderMenu::SubMenu *kdeMenu = m_vfolder->parseMenu(QStringLiteral(APPLICATIONS_MENU_NAME));
+        VFolderMenu::SubMenu *kdeMenu = m_vfolder->parseMenu(QStringLiteral("applications.menu"));
 
         KServiceGroup::Ptr entry = m_buildServiceGroupFactory->addNew(QStringLiteral("/"), kdeMenu->directoryFile, KServiceGroup::Ptr(), false);
         entry->setLayoutInfo(kdeMenu->layoutList);
@@ -385,7 +374,6 @@ bool KBuildSycoca::recreate(bool incremental)
         m_ctimeDict = new KCTimeDict;
 
         // Must be in same order as in KBuildSycoca::recreate()!
-        m_allEntries->append(KSycocaPrivate::self()->serviceTypeFactory()->allEntries());
         m_allEntries->append(KSycocaPrivate::self()->mimeTypeFactory()->allEntries());
         m_allEntries->append(KSycocaPrivate::self()->serviceGroupFactory()->allEntries());
         m_allEntries->append(KSycocaPrivate::self()->serviceFactory()->allEntries());
@@ -413,14 +401,11 @@ bool KBuildSycoca::recreate(bool incremental)
     m_newTimestamp = QDateTime::currentMSecsSinceEpoch();
     qCDebug(SYCOCA).nospace() << "Recreating ksycoca file (" << path << ", version " << KSycoca::version() << ")";
 
-    // It is very important to build the servicetype one first
-    KBuildServiceTypeFactory *buildServiceTypeFactory = new KBuildServiceTypeFactory(this);
-    d->m_serviceTypeFactory = buildServiceTypeFactory;
     KBuildMimeTypeFactory *buildMimeTypeFactory = new KBuildMimeTypeFactory(this);
     d->m_mimeTypeFactory = buildMimeTypeFactory;
     m_buildServiceGroupFactory = new KBuildServiceGroupFactory(this);
     d->m_serviceGroupFactory = m_buildServiceGroupFactory;
-    d->m_serviceFactory = new KBuildServiceFactory(buildServiceTypeFactory, buildMimeTypeFactory, m_buildServiceGroupFactory);
+    d->m_serviceFactory = new KBuildServiceFactory(buildMimeTypeFactory, m_buildServiceGroupFactory);
 
     if (build()) { // Parse dirs
         save(str); // Save database
@@ -446,19 +431,6 @@ bool KBuildSycoca::recreate(bool incremental)
             qCWarning(SYCOCA) << "ERROR writing database" << database.fileName() << database.errorString();
             return false;
         }
-
-        // Compatibility code for KF < 5.15: provide a ksycoca5 symlink after the filename change, for old apps to keep working during the upgrade
-        const QString oldSycoca = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/ksycoca5");
-        if (QFile::exists(oldSycoca)) {
-            QFile::remove(oldSycoca);
-#ifdef Q_OS_UNIX
-            if (::link(QFile::encodeName(path).constData(), QFile::encodeName(oldSycoca).constData()) != 0) {
-                QFile::copy(path, oldSycoca);
-            }
-#else
-            QFile::copy(path, oldSycoca);
-#endif
-        }
     } else {
         delete str;
         str = nullptr;
@@ -511,7 +483,7 @@ void KBuildSycoca::save(QDataStream *str)
     (*str) << m_newTimestamp;
     (*str) << QLocale().bcp47Name();
     // This makes it possible to trigger a ksycoca update for all users (KIOSK feature)
-    (*str) << calcResourceHash(QStringLiteral("kservices5"), QStringLiteral("update_ksycoca"));
+    (*str) << calcResourceHash(QStringLiteral("kservices6"), QStringLiteral("update_ksycoca"));
     (*str) << m_allResourceDirs.keys();
     for (auto it = m_allResourceDirs.constBegin(); it != m_allResourceDirs.constEnd(); ++it) {
         (*str) << it.value();
@@ -567,7 +539,6 @@ QStringList KBuildSycoca::factoryResourceDirs()
     }
     dirs = new QStringList;
     // these are all resource dirs cached by ksycoca
-    *dirs += KServiceTypeFactory::resourceDirs();
     *dirs += KMimeTypeFactory::resourceDirs();
     *dirs += KServiceFactory::resourceDirs();
 
@@ -651,7 +622,7 @@ bool KBuildSycoca::checkGlobalHeader()
 {
     // Since it's part of the filename, we are 99% sure that the locale and prefixes will match.
     const QString current_language = QLocale().bcp47Name();
-    const quint32 current_update_sig = KBuildSycoca::calcResourceHash(QStringLiteral("kservices5"), QStringLiteral("update_ksycoca"));
+    const quint32 current_update_sig = KBuildSycoca::calcResourceHash(QStringLiteral("kservices6"), QStringLiteral("update_ksycoca"));
     const QString current_prefixes = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).join(QString(QLatin1Char(':')));
 
     const KSycocaHeader header = KSycocaPrivate::self()->readSycocaHeader();
