@@ -32,7 +32,6 @@
 #include <kmimetypefactory_p.h>
 #include <kservicefactory_p.h>
 #include <kservicegroupfactory_p.h>
-#include <kservicetypefactory_p.h>
 #include <stdlib.h>
 
 #include "kbuildsycoca_p.h"
@@ -90,7 +89,6 @@ KSycocaPrivate::KSycocaPrivate(KSycoca *qq)
     , m_mmapFile(nullptr)
     , m_device(nullptr)
     , m_mimeTypeFactory(nullptr)
-    , m_serviceTypeFactory(nullptr)
     , m_serviceFactory(nullptr)
     , m_serviceGroupFactory(nullptr)
 {
@@ -103,7 +101,7 @@ KSycocaPrivate::KSycocaPrivate(KSycoca *qq)
 #else
     m_sycocaStrategy = StrategyMmap;
 #endif
-    KConfigGroup config(KSharedConfig::openConfig(), "KSycoca");
+    KConfigGroup config(KSharedConfig::openConfig(), QStringLiteral("KSycoca"));
     setStrategyFromString(config.readEntry("strategy"));
 }
 
@@ -312,23 +310,19 @@ QDataStream *&KSycocaPrivate::stream()
 
 void KSycocaPrivate::slotDatabaseChanged()
 {
-    // We don't have information anymore on what resources changed, so emit them all
-    changeList = QStringList() << QStringLiteral("services") << QStringLiteral("servicetypes") << QStringLiteral("xdgdata-mime") << QStringLiteral("apps");
-
     qCDebug(SYCOCA) << QThread::currentThread() << "got a notifyDatabaseChanged signal";
-    // KDirWatch tells us the database file changed
-    // We would have found out in the next call to ensureCacheValid(), but for
-    // now keep the call to closeDatabase, to help refcounting to 0 the old mmapped file earlier.
-    closeDatabase();
-    // Start monitoring the new file right away
-    m_databasePath = findDatabase();
+    // In case we have changed the database outselves, we have already notified the application
+    if (!m_dbLastModified.isValid() || m_dbLastModified != QFileInfo(m_databasePath).lastModified()) {
+        // KDirWatch tells us the database file changed
+        // We would have found out in the next call to ensureCacheValid(), but for
+        // now keep the call to closeDatabase, to help refcounting to 0 the old mmapped file earlier.
+        closeDatabase();
+        // Start monitoring the new file right away
+        m_databasePath = findDatabase();
 
-    // Now notify applications
-    Q_EMIT q->databaseChanged();
-
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 80)
-    Q_EMIT q->databaseChanged(changeList);
-#endif
+        // Now notify applications
+        Q_EMIT q->databaseChanged();
+    }
 }
 
 KMimeTypeFactory *KSycocaPrivate::mimeTypeFactory()
@@ -339,14 +333,6 @@ KMimeTypeFactory *KSycocaPrivate::mimeTypeFactory()
     return m_mimeTypeFactory;
 }
 
-KServiceTypeFactory *KSycocaPrivate::serviceTypeFactory()
-{
-    if (!m_serviceTypeFactory) {
-        m_serviceTypeFactory = new KServiceTypeFactory(q);
-    }
-    return m_serviceTypeFactory;
-}
-
 KServiceFactory *KSycocaPrivate::serviceFactory()
 {
     if (!m_serviceFactory) {
@@ -363,13 +349,6 @@ KServiceGroupFactory *KSycocaPrivate::serviceGroupFactory()
     return m_serviceGroupFactory;
 }
 
-// Add local paths to the list of dirs we got from the global database
-void KSycocaPrivate::addLocalResourceDir(const QString &path)
-{
-    // If any local path is more recent than the time the global sycoca was created, build a local sycoca.
-    allResourceDirs.insert(path, timeStamp);
-}
-
 // Read-write constructor - only for KBuildSycoca
 KSycoca::KSycoca(bool /* dummy */)
     : d(new KSycocaPrivate(this))
@@ -411,7 +390,6 @@ void KSycocaPrivate::closeDatabase()
 
     m_mimeTypeFactory = nullptr;
     m_serviceFactory = nullptr;
-    m_serviceTypeFactory = nullptr;
     m_serviceGroupFactory = nullptr;
 
 #if HAVE_MMAP
@@ -435,13 +413,6 @@ void KSycoca::addFactory(KSycocaFactory *factory)
     d->addFactory(factory);
 }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
-bool KSycoca::isChanged(const char *type)
-{
-    return self()->d->changeList.contains(QString::fromLatin1(type));
-}
-#endif
-
 QDataStream *KSycoca::findEntry(int offset, KSycocaType &type)
 {
     QDataStream *str = stream();
@@ -478,10 +449,6 @@ bool KSycocaPrivate::checkVersion()
     }
 }
 
-// This is now completely useless. KF6: remove
-extern KSERVICE_EXPORT bool kservice_require_kded;
-KSERVICE_EXPORT bool kservice_require_kded = true;
-
 // If it returns true, we have a valid database and the stream has rewinded to the beginning
 // and past the version number.
 bool KSycocaPrivate::checkDatabase(BehaviorsIfNotFound ifNotFound)
@@ -676,7 +643,8 @@ void KSycocaPrivate::checkDirectories()
 
 bool KSycocaPrivate::needsRebuild()
 {
-    if (!timeStamp && databaseStatus == DatabaseOK) {
+    // In case it is not open, it might be due to another process/thread having rebuild it. Thus we read the header for both the not open and ok state
+    if (!timeStamp && databaseStatus != BadVersion) {
         (void)readSycocaHeader();
     }
     // these days timeStamp is really a "bool headerFound", the value itself doesn't matter...
@@ -709,32 +677,12 @@ bool KSycocaPrivate::buildSycoca()
         qCDebug(SYCOCA) << "Still no database...";
         return false;
     }
+    Q_EMIT q->databaseChanged();
     return true;
 }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 15)
-quint32 KSycoca::timeStamp()
+QString KSycoca::absoluteFilePath()
 {
-    if (!d->timeStamp) {
-        (void)d->readSycocaHeader();
-    }
-    return d->timeStamp / 1000; // from ms to s
-}
-#endif
-
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 15)
-quint32 KSycoca::updateSignature()
-{
-    if (!d->timeStamp) {
-        (void)d->readSycocaHeader();
-    }
-    return d->updateSig;
-}
-#endif
-
-QString KSycoca::absoluteFilePath(DatabaseType type)
-{
-    Q_UNUSED(type); // GlobalDatabase concept removed in 5.61
     const QStringList paths = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
     QString suffix = QLatin1Char('_') + QLocale().bcp47Name();
 
@@ -746,23 +694,13 @@ QString KSycoca::absoluteFilePath(DatabaseType type)
 #ifdef Q_OS_WIN
         suffix.replace(QLatin1Char(':'), QLatin1Char('_'));
 #endif
-        const QString fileName = QLatin1String("ksycoca5") + suffix;
+        const QString fileName = QLatin1String("ksycoca6") + suffix;
         return QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1Char('/') + fileName;
     } else {
         return QFile::decodeName(ksycoca_env);
     }
 }
 
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 15)
-QString KSycoca::language()
-{
-    if (d->language.isEmpty()) {
-        (void)d->readSycocaHeader();
-    }
-    return d->language;
-}
-#endif
-
 QStringList KSycoca::allResourceDirs()
 {
     if (!d->timeStamp) {
